import json
from cryptography.fernet import Fernet
from cryptography.fernet import InvalidToken
import os.path
class Database:
    def __init__(self, key, filename="troll.trolldb"):
        if len(key) < 0:
            raise Exception("""Key not defined, Usage: Database("key here")""")
        try:
            self.key = bytes(key, 'utf-8')
        except:
            raise Exception("Invalid, generate a key with trolldb.genkey()")
        self.filename = filename
        if not os.path.isfile(filename):
            open(filename, 'wb').write(Fernet(self.key).encrypt(b"{}"))
        # testing key if its valid
        try:
            key = Fernet(self.key)
            str(key.decrypt(open(self.filename, 'rb').read()), 'utf-8')
        except InvalidToken:
            raise Exception("Wrong key defined.")
    def getkey(self):
        return self.key
    def getfilename(self):
        return self.filename
    def setvariable(self, name, value):
        key = Fernet(self.key)
        d = json.loads(key.decrypt(open(self.filename, 'rb').read()))
        d[name] = value
        with open(self.filename, 'wb') as a:
            a.write(Fernet(self.key).encrypt(bytes(json.dumps(d), 'utf-8')))
    def getvariable(self, name):
        if len(name) < 0:
            raise Exception("Name of the variable not defined")
        key = Fernet(self.key)
        d = json.loads(key.decrypt(open(self.filename, 'rb').read()))
        return d[name]
    def getplainjson(self):
        key = Fernet(self.key)
        return str(key.decrypt(open(self.filename, 'rb').read()), 'utf-8')

def genkey():
    gk = Fernet.generate_key()
    print(f"your generated key = {gk}, written to key.trolldb")
    open("key.trolldb", 'wb').write(gk)
    exit()
def key_by_filename(filename):
    return open(filename,'r').read()


import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="trolldb",
    version="1.0.0",
    description="JSON Database but just encrypted",
    url="https://github.com/ProYT303/trollDB",
    author="Walter#5555",
    author_email="social.proyt303@gmail.com",
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    packages=["cryptography"],
    include_package_data=True,
    install_requires=["json", "cryptography"],
)


# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.


from enum import Enum


class _Reasons(Enum):
    BACKEND_MISSING_INTERFACE = 0
    UNSUPPORTED_HASH = 1
    UNSUPPORTED_CIPHER = 2
    UNSUPPORTED_PADDING = 3
    UNSUPPORTED_MGF = 4
    UNSUPPORTED_PUBLIC_KEY_ALGORITHM = 5
    UNSUPPORTED_ELLIPTIC_CURVE = 6
    UNSUPPORTED_SERIALIZATION = 7
    UNSUPPORTED_X509 = 8
    UNSUPPORTED_EXCHANGE_ALGORITHM = 9
    UNSUPPORTED_DIFFIE_HELLMAN = 10
    UNSUPPORTED_MAC = 11


class UnsupportedAlgorithm(Exception):
    def __init__(self, message, reason=None):
        super(UnsupportedAlgorithm, self).__init__(message)
        self._reason = reason


class AlreadyFinalized(Exception):
    pass


class AlreadyUpdated(Exception):
    pass


class NotYetFinalized(Exception):
    pass


class InvalidTag(Exception):
    pass


class InvalidSignature(Exception):
    pass


class InternalError(Exception):
    def __init__(self, msg, err_code):
        super(InternalError, self).__init__(msg)
        self.err_code = err_code


class InvalidKey(Exception):
    pass
